/*
snap.c

Written by Windell Oskay, http://www.evilmadscientist.com/

Copyright 2013 Windell H. Oskay
Distributed under the terms of the GNU General Public License, please see below.

An avr-gcc program for the Atmel ATTiny2313 

Date Created:   10/13/07
Last Modified:  10/17/13
 
Purpose: Simple control of an RC servo motor for a robotic pumpkin.
Designed for "snapping" pumpkin servo output on pins B0 or B1.

The servo signal on Port B1 has a  range twice that of the one at B0, 
allowing you to quickly switch (in hardware) the servo output range
even after the chip is programmed.

LED "Eye" outputs: Pins D4, D5

Connect a 1k resistor in series with a Red, orange, or yellow LED on each output D4, D5.

More information about this project is at 
http://wiki.evilmadscientist.com/Snap-O-Lantern
http://shop.evilmadscientist.com/productsmenu/tinykitlist/659

We will run with the 8 MHz internal RC oscillator,
and with the clock prescaler turned off (divide by 1)

Count: System clock.
Available Servo PWM outputs: pins B0 - B5
Servo PWM outputs used: B0, B1 
LED outputs: Pins D4, D5

-------------------------------------------------

 Rev C, October 2013. 
 Updated signal names to reflect changes in AVR libraries.
 Initial delay after turn-on is now 1/4 of the delay, or 5 s from our default of 20s delay.
 
 
 
 

-------------------------------------------------
USAGE: How to compile and install

You will need an AVR programmer and appropriate target board to use this firmware.  

Brand new to AVR?  Start here: http://www.ladyada.net/learn/avrdevtut/index.html
Please also see: http://www.evilmadscientist.com/article.php/avrtargetboards

A makefile is provided to compile and install this program using AVR-GCC and avrdude.

To use it, follow these steps:
1. Update the header of the makefile as needed to reflect the type of AVR programmer that you use.
2. Open a terminal window and move into the directory with this file and the makefile.  
3. At the terminal enter
		make clean   <return>
		make all     <return>
		make program <return>
4. Make sure that avrdude does not report any errors.  If all goes well, the last few lines output by avrdude
should look something like this:

Reading | ################################################## | 100% 0.37s

avrdude: verifying ...
avrdude: 566 bytes of flash verified

avrdude: safemode: lfuse reads as E4
avrdude: safemode: hfuse reads as DF
avrdude: safemode: efuse reads as FF
avrdude: safemode: Fuses OK

avrdude done.  Thank you.

If you do use a different programming environment, make sure that you copy over 
the fuse settings from the makefile.

-------------------------------------------------

This code should be relatively straightforward, so not much documentation is provided.  If you'd like to ask 
questions, suggest improvements, or report success, please use the evilmadscientist forum:
http://www.evilmadscientist.com/forum/

-------------------------------------------------

 This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


*/

#include <avr/io.h>  
#include <avr/interrupt.h> 

// Global variables
volatile unsigned short BITPHASE;		// Varies over 0-5, which servo output we are using.
volatile unsigned short PulseDone;		// boolean flag


#define DelayTime 600U			// # of 1/30 s intervals to wait before snapping 
// Set to 600 for one snap every 20 seconds-- seems like a good value.



SIGNAL(TIMER1_COMPA_vect)
{
	PORTB = (1 << BITPHASE);
}

SIGNAL(TIMER1_COMPB_vect)
{
	PORTB = 0;
	PulseDone = 1;
}
	
int main (void)
{ 

asm("cli");		// DISABLE global interrupts

unsigned int TimeOffset;	// Initial Delay (1000 cycles?)
unsigned int positions[6];  // Values should range 0 - 1000. 

unsigned short i, PosTemp;
 
//uint32_t longwaittimer;
unsigned int longwaittimer;

PORTB = 0;		//All of port B should be off.
DDRB =  63U;	// #0011 1111; PB0 - PB6 are our outputs


PORTD = 0;
DDRD =	48U;	// Pin D4, D5 are our LED outputs


//Rate of MCU clock, in MHz
#define clockrate 8

TimeOffset = 1000;

// Extensible design: can handle up to 6 servo outputs:
// The position value ranges over 0 - 1000.
// position[i] PLUS TimeOffset  is the number of microseconds that the
// control signal is high; it should range from 1-2 ms.

positions[0] = 100;
positions[1] = 200;
positions[2] = 400;
positions[3] = 600;
positions[4] = 800;
positions[5] = 900;




////////////////////////////////////////////////////
//
//Timing controlled by 16-bit timer 1, a free running timer at the CPU clock rate
//
////////////////////////////////////////////////////


TCCR1A = 0;

//TCCR1A:
// Bits 7-4:  0000, Normal port operation, OC1A/B disconnected
// Bits 1,0: 00, WGM, normal mode


//TCCR1B =  1;		// Run clock
TCCR1B =  0;		// No clock

//TCCR1B:
// Bit 7: 0: Input noise cancel
// Bit 6: 0:  Input Capture Edge Select; positive edge trigger
// Bit 4,3: 0 (WGM)
// Bits 2,1,0:  001 Clock Select, use main clock w/o division
// Bits 2,1,0:  000 Clock Select: No clock; clock stopped.

TCCR1C =  0;		// CYA Only


////////////////////////////////////////////////////
// 8-bit Timer 0: For real-time waiting

// Set longwaittimer to 3/4 of longwaittimer
longwaittimer = (DelayTime) - (DelayTime >> 2); // Initial wait is 1/4 of full delay
TCNT0 = 0;

TCCR0A = 0;	// "Normal operation"
//TCCR0B = 0;// Stop clock
TCCR0B = 5;// Start timer, at Clock Rate/1024.

TIFR |= 2U;  /// Clear bit 1, Ctr 0 overflow flag by writing 1.



////////////////////////////////////////////////////


																																
BITPHASE = 0;
PulseDone = 0;

	

asm("sei");		// ENABLE global interrupts 
											
for (;;)  // main loop										
{

if (TIFR & 2U)		// Overflow:  at 256*1024 clock ticks = 262000, 30 Hz.
{
	longwaittimer++;		// incremented at 30 Hz.
	TIFR |= 2U;	// Clear overflow flag
}

/*
Plan:
1. Disable interrupts
2. Stop Counter
3. Clear PulseDone flag
4. Set Low & Hi compare values
5. Clear compare flags  (Necessary?)
6. Reset counter to zero.
7. Enable interrupts
8. Start counter
9. Wait for PulseDone flag.
10. increment phase

Repeat
*/

TIMSK = 0; // Disable timer interrupts

TCCR1B =  0;		// Stop Counter

PulseDone = 0;

//Set Low & Hi compare values:

OCR1A = TimeOffset;
OCR1B = (1000 + positions[BITPHASE])*clockrate + TimeOffset;

// Reset counter to zero.
TCNT1 = 0;

TIMSK = 96U; // enable timer OC interrupts (bit 0: overflow)

TCCR1B =  1;		// Start Counter

if ((longwaittimer < 5U) || (longwaittimer > DelayTime))
	while (PulseDone == 0) {;;}			


i = BITPHASE;
 

if (BITPHASE == 0)
{
PosTemp = positions[i];

if (longwaittimer > DelayTime)		// Wait 30 s.
{
PORTD = 48U;

	if (PosTemp < (unsigned int) 490U){
		 positions[0] = PosTemp + 1;
		 positions[1] = positions[1] + 2;
		 }
	else{
		positions[0]  = 10;
		positions[1]  = 10;
		longwaittimer = 0;
		} 
}
else
{
PORTD = 0;
		positions[0]  = 10;
		positions[1]  = 10;
		}
}

if ((++BITPHASE) > 5)
	BITPHASE = 0;

} // End main loop


	return 0;
}
